#!/usr/bin/python
#
#       AWN Radio Applet
#
#       Copyright (c) 2008, 2009, 2010 Bryan Blunt <bryan@blunt.me.uk>
#       Based on AWN Test Python Applet Copyright (c) 2007 Neil Jagdish Patel
#
#       This library is free software; you can redistribute it and/or
#       modify it under the terms of the GNU Lesser General Public
#       License as published by the Free Software Foundation; either
#       version 2 of the License, or (at your option) any later version.
#
#       This library is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#       Lesser General Public License for more details.
#
#       You should have received a copy of the GNU Lesser General Public
#       License along with this library; if not, write to the
#       Free Software Foundation, Inc., 59 Temple Place - Suite 330,
#       Boston, MA 02111-1307, USA.


import os
import gtk
import awn
from xdg import BaseDirectory
from awn.extras import awnlib

#Interface to various media players
import radiointerface

applet_name = "Radio"
applet_short_name = "radio"
applet_version = "0.10"
applet_description = "An applet to play internet radio"
applet_logo = os.path.join(os.path.dirname(__file__), "icons", "radio.svg")

USER_DIR = os.path.join(BaseDirectory.xdg_config_home,
                        'awn', 'applets', 'radio')
STATUS_FILE = os.path.join(USER_DIR, "status")
DEFAULT_ICON_DIR = os.path.join(USER_DIR, "icons")
DEFAULT_REC_DIR = os.getenv("HOME")
DEFAULT_NAMES = ["BBC Radio 4 (FM)",
                 "BBC Radio 4 (LW)",
                 "BBC Radio 5 Live",
                 "BBC Radio 5 Live Sport Extra",
                 "BBC Radio 6",
                 "BBC Radio 7",
                 "BBC World Service",
                 "XFM"]
DEFAULT_URIS = ["http://www.bbc.co.uk/radio/listen/live/r4.asx",
                "http://www.bbc.co.uk/radio/listen/live/r4lw.asx",
                "http://www.bbc.co.uk/radio/listen/live/r5l.asx",
                "http://www.bbc.co.uk/radio/listen/live/r5lsp.asx",
                "http://www.bbc.co.uk/radio/listen/live/r6.asx",
                "http://www.bbc.co.uk/radio/listen/live/r7.asx",
                "http://www.bbc.co.uk/worldservice/meta/tx/nb/live/www15.ram",
                "http://media-ice.musicradio.com/XFMMP3.m3u"]
DEFAULT_ICONS = ["",
                "",
                "",
                "",
                "",
                "",
                "",
                ""]


class RadioApplet(object):
    """ An applet to play internet radio """

    def __init__(self, applet):

        self.applet = applet
        self.applet.tooltip.set("Not playing")

        self.applet.theme.set_states({"radio": "radio"})
        self.applet.icon.theme('radio')

        self.record_icon = awn.OverlayThemedIcon("media-record")
        self.record_icon.props.scale = 0.4
        self.record_icon.props.active = False
        self.record_icon.props.gravity = gtk.gdk.GRAVITY_SOUTH_WEST
        self.applet.add_overlay(self.record_icon)

        self.play_icon = awn.OverlayThemedIcon("media-playback-start")
        self.play_icon.props.scale = 0.4
        self.play_icon.props.active = False
        self.play_icon.props.gravity = gtk.gdk.GRAVITY_SOUTH_EAST
        self.applet.add_overlay(self.play_icon)

        self.applet.get_icon().connect('drag-data-received',
                                       self.on_drop_on_app)
        self.applet.get_icon().connect("drag-motion",
                                       self.applet_drag_motion_cb)
        self.applet.get_icon().connect("drag-leave",
                                       self.applet_drag_leave_cb)

        self.get_config()
        self.ui = gtk.Builder()
        self.ui.add_from_file(os.path.join(os.path.dirname(__file__),
                                         "radio.glade"))

        self.setup_context_menu()
        self.setup_dialog_settings()
        #make sure to connect signal handlers after setting up preferences
        #dialog as populating the fields causes signals to be emitted
        self.ui.connect_signals(self)

        self.radio = radiointerface.radio(self.backend)
        if self.radio.has_backend:
            self.radio.connect_callback(self.radio_callback)
            self.setup_main_menu()
            self.setup_preset_buttons()
        else:
            self.setup_main_menu_error()
            gtk.link_button_set_uri_hook(self.open_uri)

        self.set_status()
        self.applet.dialog._Dialogs__register['about'].set_website(
                                        'https://launchpad.net/awnradioapplet')

    def open_uri(self, widget, uri):
        gtk.show_uri(gtk.gdk.screen_get_default(), uri, gtk.gdk.CURRENT_TIME)

    def setup_main_menu(self):
        self.dialog = self.applet.dialog.new("main")
        self.ui.get_object("main-vbox").reparent(self.dialog)
        self.preset_vbox = self.ui.get_object("preset-vbox")

    def setup_preset_buttons(self):
        self.dialog.hide()
        for i in self.preset_vbox.get_children():
            i.destroy()
        for i in range(0, len(self.names)):
            newbutton = gtk.Button(stock=self.names[i])
            newbutton.connect("button-press-event", self.radio_button_press, i)
            self.preset_vbox.add(newbutton)

    def setup_main_menu_error(self):
        self.dialog = self.applet.dialog.new("main")
        self.ui.get_object("error-vbox").reparent(self.dialog)

    def setup_context_menu(self):
        self.context_menu = self.applet.dialog.menu
        preferences_vbox = self.applet.dialog.new("preferences").vbox
        self.ui.get_object("prefs-notebook").reparent(preferences_vbox)

    def get_config(self):

        if "names" not in self.applet.settings:
            self.applet.settings["names"] = DEFAULT_NAMES
        if "URIs"  not in self.applet.settings:
            self.applet.settings["URIs"] = DEFAULT_URIS
        if "icons" not in self.applet.settings:
            self.icons = self.applet.settings["icons"] = DEFAULT_ICONS
        if "backend"  not in self.applet.settings:
            self.applet.settings["backend"] = "radio_mplayer"
        if "icondir"  not in self.applet.settings:
            self.icondir = self.applet.settings["icondir"] = DEFAULT_ICON_DIR
        if "recdir"  not in self.applet.settings:
            self.recdir = self.applet.settings["recdir"] = DEFAULT_REC_DIR

        self.names = self.applet.settings["names"]
        self.URIs = self.applet.settings["URIs"]
        self.icons = self.applet.settings["icons"]
        self.backend = self.applet.settings["backend"]
        self.icondir = self.applet.settings["icondir"]
        self.recdir = self.applet.settings["recdir"]

    def setup_dialog_settings(self):

        self.treeview = self.ui.get_object("preset-treeview")
        self.liststore = self.ui.get_object("preset-liststore")

        for i in range(0, min(len(self.names),
                              len(self.URIs),
                              len(self.icons))):
            self.liststore.append([self.names[i], self.URIs[i], self.icons[i]])

        self.icondir_entry = self.ui.get_object("entry-icondir")
        self.icondir_entry.set_text(self.icondir)

        self.recorddir_entry = self.ui.get_object("entry-recorddir")
        self.recorddir_entry.set_text(self.recdir)

        self.backend_combo = self.ui.get_object("backend-combo")
        self.backend_ls = self.ui.get_object("liststore-backend-combo")
        self.backend_ls.append(("Mplayer (running in background)",
                                "radio_mplayer"))
        self.backend_ls.append(("Mplayer (started on demand)",
                                "radio_mplayer_on_demand"))
        for i, row in enumerate(self.backend_ls):
            if row[1] == self.backend:
                #pass
                self.backend_combo.set_active(i)

        self.textview = self.ui.get_object("logging-textview")
        self.textbuffer = gtk.TextBuffer()
        self.textview.set_buffer(self.textbuffer)

    def backend_changed(self, combobox):
        self.backend = self.backend_ls[self.backend_combo.get_active()][1]
        self.applet.settings["backend"] = self.backend
        self.radio = radiointerface.radio(self.backend)
        if self.radio.has_backend:
            self.radio.connect_callback(self.radio_callback)
        self.set_status()

    def icondir_change(self, editable):
        self.icondir = self.icondir_entry.get_text()
        self.applet.settings["icondir"] = self.icondir

    def recdir_change(self, editable):
        self.recdir = self.recorddir_entry.get_text()
        self.applet.settings["recdir"] = self.recdir

    def edited_name(self, cell, path, new_text):
        self.liststore[path][0] = new_text
        self.get_stations_from_liststore()
        self.setup_preset_buttons()

    def edited_URI(self, cell, path, new_text):
        self.liststore[path][1] = new_text
        self.get_stations_from_liststore()

    def edited_icon(self, cell, path, new_text):
        self.liststore[path][2] = new_text
        self.get_stations_from_liststore()

    def reordered(self, model, path):
        self.get_stations_from_liststore()
        self.setup_preset_buttons()

    def add_station(self, widget, event):
        self.liststore.append(("", "", ""))
        self.get_stations_from_liststore()
        self.setup_preset_buttons()

    def delete_station(self, widget, event):
        to_del = self.treeview.get_selection().get_selected()
        to_del[0].remove(to_del[1])
        #relies on row-deleted signal handler to call setup_preset_buttons

    def get_stations_from_liststore(self):
        self.names = []
        self.URIs = []
        self.icons = []
        for row in self.liststore:
            self.names.append(row[0])
            self.URIs.append(row[1])
            self.icons.append(row[2])
        self.applet.settings["names"] = self.names
        self.applet.settings["URIs"] = self.URIs
        self.applet.settings["icons"] = self.icons

    def radio_button_press(self, widget, event, i):
        self.dialog.hide()
        if self.icons[i] == "":
            self.applet.icon.theme('radio')
        else:
            radio_icon = os.path.join(self.icondir, self.icons[i])
            try:
                self.applet.icon.file(radio_icon, size=self.applet.get_size())
            except:
                self.applet.icon.theme('radio')

        self.radio.load_radio_station(self.names[i], self.URIs[i])
        if self.radio.recording:
            self.radio.stop_record()
        self.set_status()

    def radio_stop(self, widget, event):
        self.dialog.hide()
        self.applet.icon.theme('radio')
        self.radio.radio_stop()
        self.set_status()

    def radio_record(self, widget, event):
        self.dialog.hide()
        if self.radio.recording:
            self.radio.stop_record()
        else:
            self.radio.record(self.recdir)
        self.set_status()

    def radio_callback(self, message):
        self.textbuffer.insert(self.textbuffer.get_end_iter(), message + "\n")
        self.textview.scroll_to_iter(self.textbuffer.get_end_iter(), 0.0)

    def applet_drag_motion_cb(self, widget, context, x, y, time):
        self.applet.get_effects().start(awn.EFFECT_LAUNCHING)
        return True

    def applet_drag_leave_cb(self, widget, context, time):
        self.applet.get_effects().stop(awn.EFFECT_LAUNCHING)
        return True

    def on_drop_on_app(self, obj, con, x, y, sel, tar, tim):
        self.dialog.hide()
        self.radio.load_radio_station(sel.data.strip(), sel.data.strip())
        self.set_status()

    def set_status(self):
        if not self.radio.name:
            self.applet.tooltip.set("Not playing")
        else:
            self.applet.tooltip.set(self.radio.name)

        self.play_icon.props.active = self.radio.playing
        self.record_icon.props.active = self.radio.recording

        if self.radio.recording:
            status = "recording"
        elif self.radio.playing:
            status = "playing"
        else:
            status = "stopped"
        f = open(STATUS_FILE, 'w')
        f.write('status=%s\n' % status)
        f.write('station=%s\n' % (self.radio.URI
                                  if self.radio.URI is not None else ""))
        f.close

if __name__ == "__main__":
    if not os.access(USER_DIR, os.W_OK):
        os.makedirs(USER_DIR)
    awnlib.init_start(RadioApplet, {"name": applet_name,
                                    "short": applet_short_name,
                                    "version": applet_version,
                                    "description": applet_description,
                                    "logo": applet_logo,
                                    "author": "Bryan Blunt",
                                    "copyright-year": "2008 - 2010",
                                    "authors": ["Bryan Blunt <bryan@blun.to>"],
                                    "artists": ["Bryan Blunt"]})
