# !/usr/bin/python

# Copyright (c) 2009, 2010 Bryan Blunt
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import pymplayer
import time
import os


def radio(backend="radio_mplayer"):
    return globals()[backend]()


class radio_backend(object):
    def __init__(self):
        self.name = None
        self.URI = None
        self.playing = False
        self.recording = False
        self.has_backend = False

    def _filename_for_record(self, directory):
        stripname = "".join([x for x in self.name if
                             x.isalpha() or x.isdigit()])
        fname = time.strftime('awn_radio_' + stripname +
                          '_%Y_%b_%d_%H_%M_%S.wav',
                          time.localtime(time.time()))
        return os.path.join(directory, fname)


class radio_mplayer(radio_backend):
    def __init__(self, arg=()):
        radio_backend.__init__(self)
        self._player = pymplayer.MPlayer(args=arg)
        #self._player = pymplayer.MPlayer(args=("-msglevel","all=6"))
        if self._player.start():
            self.has_backend = True
        self._recorder = None

    def __del__(self):
        # Be sure to stop the MPlayer process.
        if self._player:
            #self._player.command("quit")
            self._player.stop()
        if self._recorder:
            self._recorder.stop()

    def load_radio_station(self, name, URI):
        self.playing = True
        self.name = name
        self.URI = URI
        if self.URI[-3:] in ["ram", "asx", "m3u"]:
            self._player.command("loadlist " + self.URI)
        else:
            self._player.command("loadfile " + self.URI)

    def radio_stop(self):
        if self.playing:
            self._player.command("pause")
            self.playing = False
            self.name = None
            self.URI = None
        if self.recording:
            self.stop_record()

    def connect_callback(self, cb_func):
        self._player.handle_data = cb_func
        self._player.handle_error = cb_func

    def record(self, directory):
        if self.playing and not self.recording:
            self.recording = True
            self._recorder = pymplayer.MPlayer(args=("-ao", "pcm:file=" +
                                     self._filename_for_record(directory)))
            if not self._recorder.start():
                self._recorder = False
            if self.URI[-3:] in ["ram", "asx", "m3u"]:
                self._recorder.command("loadlist " + self.URI)
            else:
                self._recorder.command("loadfile " + self.URI)

    def stop_record(self):
        if self.recording:
            self.recording = False
            self._recorder.stop()
            self._recorder = None


class radio_mplayer_on_demand(radio_mplayer):
    def __init__(self, arg=()):
        radio_backend.__init__(self)
        self._player = pymplayer.MPlayer(args=arg)
        if self._player.start():
            self.has_backend = True
        self._player.stop()
        self._player = None
        self._recorder = None
        self._cb_func = None

    def load_radio_station(self, name, URI):
        if not self.playing:
            self._player = pymplayer.MPlayer(args=())
            self._player.start()
            self._player.handle_data = self._cb_func
            self._player.handle_error = self._cb_func

        self.playing = True
        self.name = name
        self.URI = URI
        if self.URI[-3:] in ["ram", "asx", "m3u"]:
            self._player.command("loadlist " + self.URI)
        else:
            self._player.command("loadfile " + self.URI)

    def radio_stop(self):
        if self.playing:
            self._player.stop()
            self.playing = False
            self.name = None
            self.URI = None
        if self.recording:
            self.stop_record()

    def connect_callback(self, cb_func):
        self._cb_func = cb_func
